package org.insa.graphs.algorithm.marathon;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.insa.graphs.algorithm.AbstractAlgorithm;
import org.insa.graphs.algorithm.ArcInspector;
import org.insa.graphs.algorithm.ArcInspectorFactory;
import org.insa.graphs.algorithm.AbstractSolution.Status;
import org.insa.graphs.algorithm.shortestpath.DijkstraAlgorithm;
import org.insa.graphs.algorithm.shortestpath.Label;
import org.insa.graphs.algorithm.shortestpath.ShortestPathAlgorithm;
import org.insa.graphs.algorithm.shortestpath.ShortestPathData;
import org.insa.graphs.algorithm.shortestpath.ShortestPathObserver;
import org.insa.graphs.algorithm.shortestpath.ShortestPathSolution;
import org.insa.graphs.algorithm.utils.BinaryHeap;
import org.insa.graphs.model.Arc;
import org.insa.graphs.model.Graph;
import org.insa.graphs.model.Node;
import org.insa.graphs.model.Path;
import java.util.Random;


public class MarathonAlgorithm extends ShortestPathAlgorithm {

    protected double pathCost;
    private static final int distanceMarathon = 42195; // en mètres
    public static Random rand = new Random();

    protected MarathonAlgorithm(ShortestPathData data) {
        super(data);
    }

    protected Label createLabel(Node node) {
        return new Label(node);
    }

    @Override
    public ShortestPathSolution run() {
        return (ShortestPathSolution) super.run();
    }

    @Override
    protected ShortestPathSolution doRun() {
        final ShortestPathData data = getInputData();
        ShortestPathSolution solution = null;
        final Graph graph = data.getGraph();
        final int nbNodes = graph.size();

        DijkstraAlgorithm dijkstra = new DijkstraAlgorithm(data);
        ShortestPathSolution path = dijkstra.run();

        while (path.getPath().getLength() < getDistance()) {
            // Recuperation indices
            // TODO : A MODIFIER POUR AMELIORER LA COHERENCE DU CHEMIN
            int longueurPath = path.getPath().size();
            List<Arc> pathMax = path.getPath().getArcs();
            float max = 0;
            int indiceArcToRemove = 0; //Math.abs(rand.nextInt() % (longueurPath - 1));
            for (int i = 0; i < pathMax.size(); i++) {
                if (pathMax.get(i).getLength() > max) {
                    max = pathMax.get(i).getLength();
                    indiceArcToRemove = i;
                }
            }
            
            // Recuperation Arc à supprimer
            Arc arcToRemove = path.getPath().getArcs().get(indiceArcToRemove);
            Node originArcToRemove = arcToRemove.getOrigin();
            Node destinationArcToRemove = arcToRemove.getDestination();

            // On le supprime dans une copie du tableau (getter Collections.unmodifiable dans Path.java)
            path.getPath().getArcs().remove(indiceArcToRemove);
            originArcToRemove.removeArc(arcToRemove);

            // Creations du chemin entre les 2 noeuds dont l'arc a été supprimé
            ArcInspector arcInspector = ArcInspectorFactory.getAllFilters().get(0);
            ShortestPathData newData = new ShortestPathData(graph, originArcToRemove, destinationArcToRemove, arcInspector);
            DijkstraAlgorithm newDijkstra = new DijkstraAlgorithm(newData);
            ShortestPathSolution newPath = newDijkstra.run();
            
            if (newPath.getPath() != null) {
                // Ajout du path trouvé à l'indice où on l'a enlevé
                path.getPath().getArcs().addAll(newPath.getPath().getArcs());
            }
        }
        
        return path;
    }

    

    public double getCostPath() {
        return this.pathCost;
    }

    public static int getDistance() {
        return distanceMarathon;
    }
}

